
/*  msl_alloc.c
 *
 *  Copyright (C) 2005 James Bergstra
 *
 *  This file is part of the Montreal Scientific Library.
 *
 *  The Montreal Scientific Library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  The Montreal Scientific Library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Montreal Scientific Library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#define _XOPEN_SOURCE 600
#define MIN_ALIGNMENT ((size_t)16)
#include "msl.h"

#if 0
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_vector_ulong.h>
#include <gsl/gsl_vector_complex_double.h>

#include <gsl/gsl_matrix.h>
#include <gsl/gsl_matrix_complex_double.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_multimin.h>

#endif
#include <assert.h>
#include <string.h>


gsl_block * 
msl_block_alloc(size_t size)
{
    //fprintf(stderr, "msl_block_alloc()\n");
    gsl_block * a  = (gsl_block*)malloc(sizeof(gsl_block));
    assert(a);
    a->size = size;
    void * p;
    assert(0 == posix_memalign(&p, MIN_ALIGNMENT, size * sizeof(double) ));
    a->data = (double *) p;
    return a;
}
gsl_block_complex * 
msl_block_complex_alloc(size_t size)
{
    gsl_block_complex * a  = (gsl_block_complex*)malloc(sizeof(gsl_block_complex));
    assert(a);
    a->size = size;
    void * p;
    assert(0 == posix_memalign(&p, MIN_ALIGNMENT, 2 * size * sizeof(double) ));
    a->data = (double *) p;
    return a;
}

gsl_matrix *
msl_matrix_alloc(size_t size1, size_t size2)
{
    gsl_matrix * a  = (gsl_matrix*)malloc(sizeof(gsl_matrix));
    assert(a);
    a->size1 = size1;
    a->size2 = size2;
    a->tda = size2;
    a->owner = 1;
    a->block = msl_block_alloc(size1 * size2);
    a->data = a->block->data;
    return a;
}
gsl_vector *
msl_vector_alloc(size_t size)
{
    gsl_vector * a  = (gsl_vector*)malloc(sizeof(gsl_vector));
    assert(a);
    a->size = size;
    a->stride = 1;
    a->owner = 1;
    a->block = msl_block_alloc(size);
    a->data = a->block->data;
    return a;
}

gsl_matrix_complex *
msl_matrix_complex_alloc(size_t size1, size_t size2)
{
    gsl_matrix_complex * a  = (gsl_matrix_complex*)malloc(sizeof(gsl_matrix_complex));
    assert(a);
    a->size1 = size1;
    a->size2 = size2;
    a->tda = size2;
    a->owner = 1;
    a->block = msl_block_complex_alloc(size1 * size2);
    a->data = a->block->data;
    return a;
}
gsl_vector_complex *
msl_vector_complex_alloc(size_t size)
{
    gsl_vector_complex * a  = (gsl_vector_complex*)malloc(sizeof(gsl_vector_complex));
    assert(a);
    a->size = size;
    a->stride = 1;
    a->owner = 1;
    a->block = msl_block_complex_alloc(size);
    a->data = a->block->data;
    return a;
}

gsl_block *
msl_block_realloc( gsl_block *b, size_t size )
{
    b->data = (double *) realloc( b->data, size );
    if ((size_t)b->data % MIN_ALIGNMENT)
    {
        fprintf(stderr, "msl_block_realloc() incurred a copy");
        void *p;
        assert(0 == posix_memalign(&p, MIN_ALIGNMENT, size * sizeof(double) ));
        memcpy(p, b->data, b->size * sizeof(double));
        free(b->data);
        b->data = (double*) p;
    }
    b->size = size;
    return b;
}

#if 0
void
msl_matrix_realloc( gsl_matrix * m, size_t size1, size_t size2)
{
    assert(m->owner);
    assert(m->tda == m->size2); //TODO: is this necessary?
    if (size1 * size2 > m->block->size ) {
        m->block->data = (double*) realloc( m->block->data, size1 * size2 * sizeof(double) );
        m->block->size = size1 * size2;
        if (m->block->data == NULL ) {
            fprintf(stderr, "msl_matrix_realloc(): Failed\n");
            exit(1);
        }
    }
    m->size1 = size1;
    m->size2 = size2;
    m->tda = size2;
    m->data = m->block->data;
}
void
msl_vector_realloc( gsl_vector * v, size_t size)
{
    assert(v->owner);
    assert(v->stride == 1); //TODO: maybe can lift this..
    if (v->block->size < size) {
        v->block->data = (double*)realloc(v->block->data, size * sizeof(double));
        v->block->size = size;
    }
    v->size = size;
    v->stride = 1;
    v->data = v->block->data;
}

#endif

