
/*  msl_fft.c
 *
 *  Copyright (C) 2005 James Bergstra
 *
 *  This file is part of the Montreal Scientific Library.
 *
 *  The Montreal Scientific Library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  The Montreal Scientific Library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Montreal Scientific Library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "msl.h"
#include <assert.h>

fftw_plan 
msl_dsp_matrix_row_fft( int dir, gsl_matrix_complex * spec, gsl_matrix * time, uint flags)
{
    assert(spec->size1 == time->size1);
    assert(spec->size2 == time->size2/2 + 1);
    assert( dir * dir == 1);

    //fftw_plan_many_dft_r2c( rank, n, howmany, in, inembed, istride, idist, out, onembed, ostride, odist, flags)
    //note that it deals correctly on the inside with complex array, with given
    //tda, size2
    int n = time->size2;
    int time_embed = time->tda;
    int spec_embed = spec->tda;
    return (dir == -1)
        ? fftw_plan_many_dft_r2c( 1, &n, (int)time->size1, time->data, &time_embed, 1, time_embed, (double (*)[2])spec->data, &spec_embed, 1, spec_embed, flags)
        : fftw_plan_many_dft_c2r( 1, &n, (int)spec->size1, (double (*)[2])spec->data, &spec_embed, 1, spec_embed, time->data, &time_embed, 1, time_embed, flags);
}

fftw_plan
msl_dsp_vector_fft( int dir, gsl_vector_complex *spec, gsl_vector *time, uint flags)
{
    assert(spec->size == time->size/2 + 1);
    assert(dir * dir == 1);

    //fftw_plan_many_dft_r2c( rank, n, howmany, in, inembed, istride, idist, out, onembed, ostride, odist, flags)
    //note that it deals correctly on the inside with complex array, with given
    //tda, size2
    int n = time->size;
    int time_embed = time->size * time->stride;
    int spec_embed = spec->size * spec->stride;
    return (dir == -1)
        ? fftw_plan_many_dft_r2c( 1, &n, 1, time->data, &time_embed, (int)time->stride, 0, (double (*)[2])spec->data, &spec_embed, (int)spec->stride, 0, flags)
        : fftw_plan_many_dft_c2r( 1, &n, 1, (double (*)[2])spec->data, &spec_embed, (int)spec->stride, 0, time->data, &time_embed, (int)time->stride, 0, flags);
}



