
/*  msl_icblas.c
 *
 *  Copyright (C) 2005 James Bergstra
 *
 *  This file is part of the Montreal Scientific Library.
 *
 *  The Montreal Scientific Library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  The Montreal Scientific Library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Montreal Scientific Library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "msl.h"

void msl_cblas_dswap(const int N, double *X, const int incX, double *Y, const int incY)
{
    int i;
    double t;
    for (i = 0; i < N; ++i)
    {
        t = X[i*incX];
        X[i*incX] = Y[i*incY];
        Y[i*incY] = t;
    }
}
void msl_cblas_dcopy(const int N, const double *X, const int incX, double *Y, const int incY)
{
    int i;
    for (i = 0; i < N; ++i)
    {
        Y[i*incY] = X[i*incX];
    }
}
void msl_cblas_daxpy(const int N, const double alpha, const double *X, const int incX, double *Y, const int incY)
{
    int i;
    for (i = 0; i < N; ++i)
    {
        Y[i*incY] += alpha * X[i*incX];
    }
}
void msl_cblas_dscal(const int N, const double alpha, double *X, const int incX)
{
    int i;
    for (i = 0; i < N; ++i)
    {
        X[i*incX] *= alpha;
    }
}
double msl_cblas_ddot(const int N, const double *X, const int incX, const double *Y, const int incY)
{
    int i;
    double rval = 0;
    for (i = 0; i < N; ++i)
    {
        rval += X[i*incX] * Y[i*incY];
    }
    return rval;
}
double msl_cblas_dnrm2(const int N, const double *X, const int incX)
{
    int i;
    double rval = 0;
    for (i = 0; i < N; ++i)
    {
        rval += X[i*incX] * X[i*incX];
    }
    return sqrt(rval);
}
double msl_cblas_dasum(const int N, const double *X, const int incX)
{
    int i;
    double rval = 0;
    for (i = 0; i < N; ++i)
    {
        rval += X[i*incX];
    }
    return rval;
}
CBLAS_INDEX msl_cblas_idamax(const int N, const double *X, const int incX)
{
    CBLAS_INDEX rval = 0;
    double m = *X;
    int i;
    for (i = 1; i < N; ++i)
    {
        if (X[i*incX] > m) {
            m = X[i*incX];
            rval = i;
        }
    }
    return rval;
}

