
/*  msl_nnet.c
 *
 *  Copyright (C) 2005 James Bergstra
 *
 *  This file is part of the Montreal Scientific Library.
 *
 *  The Montreal Scientific Library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  The Montreal Scientific Library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Montreal Scientific Library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <gsl/gsl_combination.h>
#include <gsl/gsl_permutation.h>
#include <gsl/gsl_randist.h>
#include <assert.h>
#include "msl.h"

static void
msl_combination_not(gsl_combination *notA, const gsl_combination *A)
{
    size_t i, j = 0;
    for ( i = 0; i < A->n; ++i) notA->data[i] = i + 1;
    for ( i = 0; i < A->k; ++i) notA->data[ A->data[i] ] = 0;
    for ( i = 0; i < A->n; ++i) 
    {
        if (notA->data[i])
        {
            notA->data[j++] = notA->data[i] - 1;
        }
    }
    notA->n = notA->n;
    notA->k = j;
}
static void 
nfold_idx(gsl_vector_ulong * v, const gsl_combination * c, const gsl_permutation * p)
{
    size_t fsize = p->size / c->n;
    size_t i;
    v->size = c->k * fsize;
    for (i = 0; i < c->k; ++i)
    {
        size_t j;
        for (j = 0; j < p->size / c->n; ++j)
        {
            MSL_VECTOR_AT(v, i * fsize + j) = p->data[ c->data[i] * fsize + j];
        }
    }
}

double 
msl_nnet_arch_test_kfold( 
        msl_nnet_arch * arch,
        const gsl_matrix * S, 
        const gsl_matrix * T, 
        const size_t nfolds, 
        gsl_rng * r)
{
    assert(S);
    //for now...
    assert(T);
    assert(r);
    assert( S->size1 == T->size1 );
    assert( S->size1 >= nfolds );

    size_t fold = 0;
    size_t slice = 0;
    double eval_one = 0;
    double eval_sum = 0;
    double eval_num = 0;

    gsl_permutation * idxmaster = gsl_permutation_calloc( S->size1 ); //inits to identity

    gsl_vector_ulong * idx_tr = gsl_vector_ulong_alloc(S->size1 / nfolds * (nfolds - 2));
    gsl_vector_ulong * idx_va = gsl_vector_ulong_alloc(S->size1 / nfolds);
    gsl_vector_ulong * idx_te = gsl_vector_ulong_alloc(S->size1 / nfolds);

    gsl_combination * folds_tr = gsl_combination_calloc(nfolds, nfolds - 2);
    gsl_combination * folds_nt = gsl_combination_alloc( nfolds, nfolds );
    gsl_combination folds_val = {nfolds, 1, &folds_nt->data[0]};
    gsl_combination folds_tes = {nfolds, 1, &folds_nt->data[1]};

    gsl_multimin_function_fdf * fdf_tr = arch->alloc_min(arch->aparams, S->size2, T->size2, (S->size1 / nfolds) * (nfolds - 2) );
    gsl_multimin_function     * fun_va = arch->alloc_alt(arch->aparams, S->size2, T->size2, (S->size1 / nfolds) * (1) );
    gsl_multimin_function     * fun_te = arch->alloc_alt(arch->aparams, S->size2, T->size2, (S->size1 / nfolds) * (1) );

    gsl_ran_shuffle(r, idxmaster->data, S->size1, sizeof(size_t));
    msl_combination_not( folds_nt, folds_tr);

    gsl_vector * wts = gsl_vector_alloc( fdf_tr->n );

    gsl_matrix_view gmv;
    
    for (fold = 0; fold < nfolds * (nfolds - 1)/2; ++fold)
    {
        for (slice = 0; slice < 2; ++slice)
        {
            assert(folds_tes.data[0] != folds_val.data[0]);

            nfold_idx( idx_tr, folds_tr, idxmaster );
            nfold_idx( idx_va, &folds_val, idxmaster );
            nfold_idx( idx_te, &folds_tes, idxmaster );

            gmv = arch->view_s(fdf_tr->params); msl_matrix_memcpy_rows( &gmv.matrix, S, idx_tr);
            gmv = arch->view_s(fun_va->params); msl_matrix_memcpy_rows( &gmv.matrix, S, idx_va);
            gmv = arch->view_s(fun_te->params); msl_matrix_memcpy_rows( &gmv.matrix, S, idx_te);

            gmv = arch->view_t(fdf_tr->params); msl_matrix_memcpy_rows( &gmv.matrix, T, idx_tr);
            gmv = arch->view_t(fun_va->params); msl_matrix_memcpy_rows( &gmv.matrix, T, idx_va);
            gmv = arch->view_t(fun_te->params); msl_matrix_memcpy_rows( &gmv.matrix, T, idx_te);

            arch->reset(fdf_tr->params);
            arch->reset(fun_va->params);
            arch->reset(fun_te->params);

#if 0
    size_t  i;
    fprintf(stderr, "learning again()\n");
    gsl_vector_ulong_fprintf(stderr, idx_tr, "%lu");
    for ( i = 0; i < folds_tr->k; ++i) fprintf(stderr, "%lu\t", folds_tr->data[i] ); fprintf(stderr, "\n");
    //for ( i = 0; i < folds_nt->k; ++i) fprintf(stderr, "%lu\t", folds_nt->data[i]);fprintf(stderr, "\n");

    char * line = NULL;
    size_t llen = 0;
    fprintf(stderr, "learning mats:()\n");
    msl_matrix_print_spreadsheet(&line, &llen, &net_tr->S.matrix); fprintf(stderr, "%s\n", line);
    //msl_matrix_print_spreadsheet(&line, &llen, &net_va->S.matrix); fprintf(stderr, "%s\n", line);
    //msl_matrix_print_spreadsheet(&line, &llen, &net_te->S.matrix); fprintf(stderr, "%s\n", line);
#endif

            arch->minimize(fdf_tr, fun_va, wts); //the expensive call :P

            eval_one = GSL_MULTIMIN_FN_EVAL(fun_te, wts); //the cheap one

            eval_sum += eval_one;
            eval_num += 1.0;

            double t = folds_nt->data[0];
            folds_nt->data[0] = folds_nt->data[1];
            folds_nt->data[1] = t;

            fprintf(stdout, "eval = %lf\n", eval_one);
        }
        
        gsl_combination_next( folds_tr);
        msl_combination_not( folds_nt, folds_tr );
    }

    gsl_vector_free( wts );
    arch->free_min( fdf_tr );
    arch->free_alt( fun_va );
    arch->free_alt( fun_te );
    gsl_combination_free(folds_tr);
    gsl_combination_free(folds_nt);
    gsl_vector_ulong_free( idx_tr );
    gsl_vector_ulong_free( idx_va );
    gsl_vector_ulong_free( idx_te );
    gsl_permutation_free(idxmaster);

    return eval_sum / eval_num;
}





#if 0
double  //adaboost.m2 pseudo-loss
msl_matrix_AdaBoostM2_pseudoloss( const gsl_matrix *H, const gsl_matrix * D, const std::vector<size_t> Y )
{
    assert(H.sameDim(D));
    double rval = 0;

    for (size_t i = 0; i < H._s1; ++i)
    {
        for (size_t l = 0; l < H._s2; ++l) 
        {
            if ( l != Y[i] )
            {
                rval += MSL_MATRIX_AT(D,i,l) * (1.0 - MSL_MATRIX_AT(H,i,Y[i]) + MSL_MATRIX_AT(H,i,l));
            }
        }
    }
    return 0.5 * rval;
}
#endif

