
/*  msl_sndfile.c
 *
 *  Copyright (C) 2005 James Bergstra
 *
 *  This file is part of the Montreal Scientific Library.
 *
 *  The Montreal Scientific Library is free software; you can redistribute it
 *  and/or modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of the License,
 *  or (at your option) any later version.
 *
 *  The Montreal Scientific Library is distributed in the hope that it will be
 *  useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along with
 *  the Montreal Scientific Library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include "msl.h"
#include <sndfile.h>
#include <string.h>

int
msl_dsp_block_load_pcm( gsl_block * pcm, ulong * samplerate, size_t *nchannels, const char * path, const int resize )
{
    long int framestoread = 0;
    SF_INFO info;
    info.format = 0;
    SNDFILE * sf = sf_open(path, SFM_READ, &info);
    if (NULL == sf) return -1;
    *samplerate = info.samplerate;
    if (resize)
    {
        msl_block_realloc( pcm, (size_t) (info.channels * info.frames) );
        framestoread = info.frames;
    }
    else
    {
        framestoread = GSL_MIN( (size_t)info.frames, pcm->size / info.channels );
        memset(pcm->data + framestoread * info.channels, 0, sizeof(double) * (pcm->size - framestoread * info.channels));
    }
    if ( framestoread != sf_readf_double(sf, pcm->data, framestoread )) {
        sf_close(sf);
        return -2;
    }
    sf_close(sf);
    *nchannels = info.channels;
    return 0;
}

